#include <stdio.h>
#include <util/delay.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

/* CPU frequency */
#define F_CPU 16000000UL

// ISR
#define SUSPEND cli();
#define RESUME sei();

// serial communication library
#include "uart.h"
#include "uart.c"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar,
_FDEV_SETUP_RW);


// the usual
#define begin {
#define end }

#define READ(U, N) ((U) >> (N) & 1u)
#define SET(U, N) ((void)((U) |= 1u << (N)))
#define CLR(U, N) ((void)((U) &= ~(1u << (N))))
#define FLIP(U, N) ((void)((U) ^= 1u << (N))) 


#define Pushedset 1 
#define MaybePushset 2
#define Pushedfinal 3
#define MaybePushfinal 4
#define Pushedpull 5
#define MaybePushpull 6
#define Waiting 7
#define mayberel 8

void initialize(void);

//interupt for ADC convertion
ISR(ADC_vect)
begin
	ADCSRA |=1<<ADSC;
end


int main(void)
begin
initialize();
int step;
int initial = 2226, final = 2226, delta = 0;
int PushState = Pushedset;
int PreState = Pushedset;
int n=0;

while(1)
begin
 
switch (PushState) //state machine begin
	begin
		case MaybePushset:    //debounce for initial setting state
			if (~PINB & 0x01) 
				PushState = Pushedset;   
			else 
				PushState = PreState;
			break;	
		case Pushedset:      //give the ADC value to PWM output
			begin
				//ADMUX = "ch0";
				ADMUX = 0x60;
				//map the ADC value to 770~2260
				OCR1A = (int)(ADCH * 5.71) +770; 
				initial = OCR1A;
				delta = final - initial;
				if (~PINB & 0x04)  
					{
						PushState = MaybePushpull;
						PreState = Pushedset;
					}
				else if (~PINB & 0x02)
					{
						PushState = MaybePushfinal;
						PreState = Pushedset;
					}	
				else
					PushState = Pushedset; 
			end				
			break;
		case MaybePushfinal:  //debounce for final position setting
			if (~PINB & 0x02) 
				PushState = Pushedfinal;   
			else 
				PushState=PreState;
			break;
		case Pushedfinal:    //give another ADC value to PWM output
			begin
				//ADMUX = "ch1";
				ADMUX = 0x61;				
				OCR1A = (int)(ADCH * 5.71) +770;
				//fprintf(stdout,"%d  \n\r",ADCH);
				final = OCR1A; 
				delta = final - initial; //calculate the distance
				if (~PINB & 0x01) 
					{	
						PushState = MaybePushset;
						PreState = Pushedfinal;
					}
				else if (~PINB & 0x04)
					{
						PushState = MaybePushpull;
						PreState = Pushedfinal;
					}
				else
					PushState = Pushedfinal;
			end
			break;
		case MaybePushpull:  // debounce for pulling state
			_delay_ms(10);
			if (~PINB & 0x04)//If pulling, then go step to pull according to the speed
			{ 
				
				PushState = Waiting; 
				ADMUX = 0x62;

				SET(PORTD,0);
				_delay_ms(5);
				CLR(PORTD,0);

				step = (int)((255-ADCH)/50+1);
				n = (int)delta/(-step);
				if(n<0)
				{
					n=-n;
					step = -step;
				}
				while(n!=0)
				{
					OCR1A = OCR1A - step;
					n--;				
					if(OCR1A >= 2226)  //protect the servo from overflow
						OCR1A = 2226;
					else if(OCR1A <= 770)
						OCR1A = 770;
					_delay_ms(3);
					//fprintf(stdout,"%d \n\r",n);
				}
				

			}
			else 
				PushState=PreState;
			break;	
		case Pushedpull:    //the state to transfer to another state
			begin
				
				delta = final - initial;
				if (~PINB & 0x01) 
					{	
						PushState = MaybePushset;
						PreState = Pushedpull;
					}
				else if (~PINB & 0x02)
					{
						PushState = MaybePushfinal;
						PreState = Pushedpull;
					}
				else if (~PINB & 0x04)
				{
					PushState = MaybePushpull;
					PreState = Pushedpull;
					}
				else
					PushState = Pushedpull;
				
			end
			break;		

		case Waiting:    //wait until release the button
			if (~PINB & 0x04)	
				PushState = Waiting;
			else
				PushState = Pushedpull;	
			break;

	end

end
end

void initialize(void)
begin
	// PortA ADC input, PortD 4,5 PWM output
	DDRA = 0;
	DDRD =(1<<PIND0) | (1<<PIND4) | (1<<PIND5);
	DDRB = 0;
	PORTB = 0xff;
	//USE TIMER 1 for MOTOR CONTROL
	// 8 prescaler to slow down the PWM 
	// divide by 8  and set to Phase and Frequency Correct
	TCCR1B = 2 | (1<<WGM13);	// turn on PWM
	// turn on fast PWM and OC1A outputs
	TCCR1A = (1<<COM1A1) | (1<<COM1B1) ;
	OCR1A = 1500 ; // controls actual movement of servo
	ICR1 = 20000; // frequency = fclk/(2*prescaler*ICR1) = 50Hz
	//can change OCR1A between 770 and 2230 to obtain 1-2ms high pulses
	
	//Configure the ADC using ADC0
	ADCSRA |= (1<<ADPS2) | (1<<ADIE) | (1<<ADEN); // ADC prescaler is 16
	ADMUX = (1<<ADLAR) | (1<<REFS0);
	
	// turn on all ISRs
	sei() ;
   
	ADCSRA |= 1<<ADSC;
   
	//init the UART -- trt_uart_init() is in trtUart.c
	//trt_uart_init();
    	uart_init();
	stdout = stdin = stderr = &uart_str;
	fprintf(stdout,"\n\r TRT 9feb2009\n\r\n\r");
	
end

